@@ -4,6 +4,8 @@
 
 #include "Attr.h"
 #include "Expr.h"
+#include "Desc.h"
+#include "Val.h"
 #include "threading/SerialTypes.h"
 
 const char* attr_name(attr_tag t)
@@ -14,7 +16,7 @@ const char* attr_name(attr_tag t)
 		"&read_expire", "&write_expire", "&create_expire",
 		"&raw_output", "&priority",
 		"&group", "&log", "&error_handler", "&type_column",
-		"(&tracked)", "&deprecated",
+		"(&tracked)", "&on_change", "&deprecated",
 	};
 
 	return attr_names[int(t)];
@@ -164,10 +166,8 @@ void Attributes::AddAttr(Attr* attr)
 	if ( ! attrs )
 		attrs = new attr_list(1);
 
-	if ( ! attr->RedundantAttrOkay() )
-		// We overwrite old attributes by deleting them first.
-		RemoveAttr(attr->Tag());
-
+	// We overwrite old attributes by deleting them first.
+	RemoveAttr(attr->Tag());
 	attrs->push_back(attr);
 	Ref(attr);
 
@@ -290,9 +290,9 @@ void Attributes::CheckAttr(Attr* a)
 		{
 		// &default is allowed for global tables, since it's used in initialization
 		// of table fields. it's not allowed otherwise.
-		if ( global_var && ! type->IsSet() && type->Tag() != TYPE_TABLE )
+		if ( global_var && ! type->IsTable() )
 			{
-			Error("&default is not valid for global variables");
+			Error("&default is not valid for global variables except for tables");
 			break;
 			}
 
@@ -374,11 +374,21 @@ void Attributes::CheckAttr(Attr* a)
 			{
 			// &default applies to record field.
 
-			if ( same_type(atype, type) ||
-			     (atype->Tag() == TYPE_TABLE && atype->AsTableType()->IsUnspecifiedTable()) )
+			if ( same_type(atype, type) )
 				// Ok.
 				break;
 
+			if ( (atype->Tag() == TYPE_TABLE && atype->AsTableType()->IsUnspecifiedTable()) )
+				{
+				Expr* e = a->AttrExpr();
+
+				if ( check_and_promote_expr(e, type) )
+					{
+					a->SetAttrExpr(e);
+					break;
+					}
+				}
+
 			// Table defaults may be promotable.
 			if ( ytype && ytype->Tag() == TYPE_RECORD &&
 			     atype->Tag() == TYPE_RECORD &&
@@ -397,7 +407,7 @@ void Attributes::CheckAttr(Attr* a)
 		{
 		if ( type->Tag() != TYPE_TABLE )
 			{
-			Error("expiration only applicable to tables");
+			Error("expiration only applicable to sets/tables");
 			break;
 			}
 
@@ -474,6 +484,71 @@ void Attributes::CheckAttr(Attr* a)
 		}
 		break;
 
+	case ATTR_ON_CHANGE:
+		{
+		if ( type->Tag() != TYPE_TABLE )
+			{
+			Error("&on_change only applicable to tables");
+			break;
+			}
+
+		const Expr* change_func = a->AttrExpr();
+
+		if ( change_func->Type()->Tag() != TYPE_FUNC || change_func->Type()->AsFuncType()->Flavor() != FUNC_FLAVOR_FUNCTION )
+			Error("&on_change attribute is not a function");
+
+		const FuncType* c_ft = change_func->Type()->AsFuncType();
+
+		if ( c_ft->YieldType()->Tag() != TYPE_VOID )
+			{
+			Error("&on_change must not return a value");
+			break;
+			}
+
+		const TableType* the_table = type->AsTableType();
+
+		if ( the_table->IsUnspecifiedTable() )
+			break;
+
+		const type_list* args = c_ft->ArgTypes()->Types();
+		const type_list* t_indexes = the_table->IndexTypes();
+		if ( args->length() != ( type->IsSet() ? 2 : 3 ) + t_indexes->length() )
+			{
+			Error("&on_change function has incorrect number of arguments");
+			break;
+			}
+
+		if ( ! same_type((*args)[0], the_table->AsTableType()) )
+			{
+			Error("&on_change: first argument must be of same type as table");
+			break;
+			}
+
+		// can't check exact type here yet - the data structures don't exist yet.
+		if ( (*args)[1]->Tag() != TYPE_ENUM )
+			{
+			Error("&on_change: second argument must be a TableChange enum");
+			break;
+			}
+
+		for ( int i = 0; i < t_indexes->length(); i++ )
+			{
+			if ( ! same_type((*args)[2+i], (*t_indexes)[i]) )
+				{
+				Error("&on_change: index types do not match table");
+				break;
+				}
+			}
+
+		if ( ! type->IsSet() )
+			if ( ! same_type((*args)[2+t_indexes->length()], the_table->YieldType()) )
+				{
+				Error("&on_change: value type does not match table");
+				break;
+				}
+		}
+		break;
+
 	case ATTR_TRACKED:
 		// FIXME: Check here for global ID?
 		break;
