@@ -135,6 +135,10 @@ Val* Val::DoClone(CloneState* state)
 			return Ref();
 			}
 
+		if ( type->Tag() == TYPE_TYPE )
+			// These are immutable, essentially.
+			return Ref();
+
 		// Fall-through.
 
 	default:
@@ -1326,8 +1330,8 @@ static void find_nested_record_types(BroType* t, std::set<RecordType*>* found)
 		return;
 	case TYPE_LIST:
 		{
-		for ( auto& t : *t->AsTypeList()->Types() )
-			find_nested_record_types(t, found);
+		for ( auto& type : *t->AsTypeList()->Types() )
+			find_nested_record_types(type, found);
 		}
 		return;
 	case TYPE_FUNC:
