@@ -1,8 +1,9 @@
 // See the file "COPYING" in the main distribution directory for copyright.
 
+#include "KRB.h"
+
 #include <unistd.h>
 
-#include "KRB.h"
 #include "types.bif.h"
 #include "events.bif.h"
 
@@ -72,7 +73,7 @@ void KRB_Analyzer::Done()
 	}
 
 void KRB_Analyzer::DeliverPacket(int len, const u_char* data, bool orig,
-				 uint64 seq, const IP_Hdr* ip, int caplen)
+				 uint64_t seq, const IP_Hdr* ip, int caplen)
 	{
 	Analyzer::DeliverPacket(len, data, orig, seq, ip, caplen);
 
