@@ -1,8 +1,8 @@
 #include "zeek-config.h"
 
-#include "NetVar.h"
 #include "MIME.h"
-#include "Event.h"
+#include "NetVar.h"
+#include "Base64.h"
 #include "Reporter.h"
 #include "digest.h"
 #include "file_analysis/Manager.h"
@@ -537,6 +537,8 @@ MIME_Entity::MIME_Entity(MIME_Message* output_message, MIME_Entity* parent_entit
 	message = output_message;
 	if ( parent )
 		content_encoding = parent->ContentTransferEncoding();
+
+	want_all_headers = (bool)mime_all_headers;
 	}
 
 void MIME_Entity::init()
@@ -569,6 +571,7 @@ void MIME_Entity::init()
 
 	message = 0;
 	delay_adding_implicit_CRLF = false;
+	want_all_headers = false;
 	}
 
 MIME_Entity::~MIME_Entity()
@@ -744,7 +747,11 @@ void MIME_Entity::FinishHeader()
 		{
 		ParseMIMEHeader(h);
 		SubmitHeader(h);
-		headers.push_back(h);
+
+		if ( want_all_headers )
+			headers.push_back(h);
+		else
+			delete h;
 		}
 	else
 		delete h;
@@ -829,7 +836,8 @@ int MIME_Entity::ParseContentEncodingField(MIME_Header* h)
 		return 0;
 		}
 
-	content_encoding_str = new BroString((const u_char*)enc.data, enc.length, 1);
+	delete content_encoding_str;
+	content_encoding_str = new BroString((const u_char*)enc.data, enc.length, true);
 	ParseContentEncoding(enc);
 
 	if ( need_to_parse_parameters )
@@ -894,6 +902,7 @@ int MIME_Entity::ParseFieldParameters(int len, const char* data)
 				}
 
 			data_chunk_t vd = get_data_chunk(val);
+			delete multipart_boundary;
 			multipart_boundary = new BroString((const u_char*)vd.data,
 			                                   vd.length, 1);
 			}
