@@ -24,7 +24,7 @@ NTP_Analyzer::~NTP_Analyzer()
 	}
 
 void NTP_Analyzer::DeliverPacket(int len, const u_char* data, bool orig,
-                                 uint64 seq, const IP_Hdr* ip, int caplen)
+                                 uint64_t seq, const IP_Hdr* ip, int caplen)
 	{
 	Analyzer::DeliverPacket(len, data, orig, seq, ip, caplen);
 
