@@ -1,13 +1,13 @@
 // See the file "COPYING" in the main distribution directory for copyright.
 
-#include <algorithm>
-#include <vector>
-
 #include "zeek-config.h"
+#include "NFS.h"
+
+#include <utility>
+#include <vector>
 
 #include "NetVar.h"
 #include "XDR.h"
-#include "NFS.h"
 #include "Event.h"
 
 #include "events.bif.h"
@@ -19,7 +19,7 @@ int NFS_Interp::RPC_BuildCall(RPC_CallInfo* c, const u_char*& buf, int& n)
 	if ( c->Program() != 100003 )
 		Weird("bad_RPC_program", fmt("%d", c->Program()));
 
-	uint32 proc = c->Proc();
+	uint32_t proc = c->Proc();
 	// The call arguments, depends on the call type obviously ...
 	Val *callarg = 0;
 
