@@ -1,9 +1,9 @@
-#ifndef packet_h
-#define packet_h
+#pragma once
 
-#include "Desc.h"
-#include "IP.h"
-#include "NetVar.h"
+#include <string>
+
+#include <stdint.h>
+#include <sys/types.h> // for u_char
 
 #if defined(__OpenBSD__)
 #include <net/bpf.h>
@@ -12,6 +12,11 @@ typedef struct bpf_timeval pkt_timeval;
 typedef struct timeval pkt_timeval;
 #endif
 
+class Val;
+class ODesc;
+class IP_Hdr;
+class RecordVal;
+
 /**
  * The Layer 3 type of a packet, as determined by the parsing code in Packet.
  */
@@ -49,8 +54,8 @@ public:
 	 * @param tag A textual tag to associate with the packet for
 	 * differentiating the input streams.
 	 */
-	Packet(int link_type, pkt_timeval *ts, uint32 caplen,
-	       uint32 len, const u_char *data, int copy = false,
+	Packet(int link_type, pkt_timeval *ts, uint32_t caplen,
+	       uint32_t len, const u_char *data, int copy = false,
 	       std::string tag = std::string(""))
 	           : data(0), l2_src(0), l2_dst(0)
 	       {
@@ -97,8 +102,8 @@ public:
 	 * @param tag A textual tag to associate with the packet for
 	 * differentiating the input streams.
 	 */
-	void Init(int link_type, pkt_timeval *ts, uint32 caplen,
-		uint32 len, const u_char *data, int copy = false,
+	void Init(int link_type, pkt_timeval *ts, uint32_t caplen,
+		uint32_t len, const u_char *data, int copy = false,
 		std::string tag = std::string(""));
 
 	/**
@@ -114,8 +119,7 @@ public:
 	 * Interprets the Layer 3 of the packet as IP and returns a
 	 * correspondign object.
 	 */
-	const IP_Hdr IP() const
-		{ return IP_Hdr((struct ip *) (data + hdr_size), false); }
+	const IP_Hdr IP() const;
 
 	/**
 	 * Returns a \c raw_pkt_hdr RecordVal, which includes layer 2 and
@@ -148,9 +152,9 @@ public:
 	double time;			/// Timestamp reconstituted as float
 	pkt_timeval ts;			/// Capture timestamp
 	const u_char* data;		/// Packet data.
-	uint32 len;			/// Actual length on wire
-	uint32 cap_len;			/// Captured packet length
-	uint32 link_type;		/// pcap link_type (DLT_EN10MB, DLT_RAW, etc)
+	uint32_t len;			/// Actual length on wire
+	uint32_t cap_len;			/// Captured packet length
+	uint32_t link_type;		/// pcap link_type (DLT_EN10MB, DLT_RAW, etc)
 
 	// These are computed from Layer 2 data. These fields are only valid if
 	// Layer2Valid() returns true.
@@ -158,7 +162,7 @@ public:
 	/**
 	 * Layer 2 header size. Valid iff Layer2Valid() returns true.
 	 */
-	uint32 hdr_size;
+	uint32_t hdr_size;
 
 	/**
 	 * Layer 3 protocol identified (if any). Valid iff Layer2Valid()
@@ -170,7 +174,7 @@ public:
 	 * If layer 2 is Ethernet, innermost ethertype field. Valid iff
 	 * Layer2Valid() returns true.
 	 */
-	uint32 eth_type;
+	uint32_t eth_type;
 
 	/**
 	 * Layer 2 source address. Valid iff Layer2Valid() returns true.
@@ -187,13 +191,13 @@ public:
 	 * (Outermost) VLAN tag if any, else 0. Valid iff Layer2Valid()
 	 * returns true.
 	 */
-	uint32 vlan;
+	uint32_t vlan;
 
 	/**
 	 * (Innermost) VLAN tag if any, else 0. Valid iff Layer2Valid()
 	 * returns true.
 	 */
-	uint32 inner_vlan;
+	uint32_t inner_vlan;
 
 private:
 	// Calculate layer 2 attributes. Sets
@@ -212,5 +216,3 @@ private:
 	// True if L2 processing succeeded.
 	bool l2_valid;
 };
-
-#endif // packet_h
