@@ -14,6 +14,7 @@
 #include "../Func.h"
 #include "../Event.h"
 #include "../util.h"
+#include "../input.h"
 
 using namespace plugin;
 
@@ -159,19 +160,15 @@ bool Manager::ActivateDynamicPluginInternal(const std::string& name, bool ok_if_
 		return false;
 		}
 
-	if ( m->second == "" )
-		// Already activated.
-		return true;
-
-	std::string dir = m->second + "/";
-
-	if ( dir.empty() )
+	if ( m->second.empty() )
 		{
 		// That's our marker that we have already activated this
 		// plugin. Silently ignore the new request.
 		return true;
 		}
 
+	std::string dir = m->second + "/";
+
 	DBG_LOG(DBG_PLUGINS, "Activating plugin %s", name.c_str());
 
 	// Add the "scripts" and "bif" directories to ZEEKPATH.
@@ -193,6 +190,7 @@ bool Manager::ActivateDynamicPluginInternal(const std::string& name, bool ok_if_
 		if ( is_file(init) )
 			{
 			DBG_LOG(DBG_PLUGINS, "  Loading %s", init.c_str());
+			warn_if_legacy_script(init);
 			scripts_to_load.push_back(init);
 			break;
 			}
@@ -206,6 +204,7 @@ bool Manager::ActivateDynamicPluginInternal(const std::string& name, bool ok_if_
 		if ( is_file(init) )
 			{
 			DBG_LOG(DBG_PLUGINS, "  Loading %s", init.c_str());
+			warn_if_legacy_script(init);
 			scripts_to_load.push_back(init);
 			break;
 			}
@@ -218,6 +217,7 @@ bool Manager::ActivateDynamicPluginInternal(const std::string& name, bool ok_if_
 		if ( is_file(init) )
 			{
 			DBG_LOG(DBG_PLUGINS, "  Loading %s", init.c_str());
+			warn_if_legacy_script(init);
 			scripts_to_load.push_back(init);
 			break;
 			}
@@ -275,6 +275,8 @@ bool Manager::ActivateDynamicPluginInternal(const std::string& name, bool ok_if_
 
 			DBG_LOG(DBG_PLUGINS, "  Loaded %s", path);
 			}
+
+		globfree(&gl);
 		}
 
 	else
@@ -476,9 +478,9 @@ Manager::bif_init_func_map* Manager::BifFilesInternal()
 	return bifs;
 	}
 
-Plugin* Manager::LookupPluginByPath(std::string path)
+Plugin* Manager::LookupPluginByPath(std::string_view _path)
 	{
-	path = normalize_path(path);
+	auto path = normalize_path(_path);
 
 	if ( is_file(path) )
 		path = SafeDirname(path).result;
